import { useRef, useState, useEffect, createElement } from 'react';
import { newInstance } from '@jsplumb/browser-ui';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

// define some const -------------------------------------------
var NODE_ID_ATTR = '_node_id';
var NODE_ID_LEN = 10;
var NODE_PARENT_ATTR = '_node_parent';
var DEFAULT_TREE_OPTION = {
  canHighlight: true
};
var DEFAULT_CONNECTION_OPTION = {
  endpoint: 'Dot',
  // anchor: 'AutoDefault',
  // dragOptions: { cursor: 'pointer', zIndex: 2000 },
  // paintStyle: { stroke: '#666' },
  // endpointHoverStyle: { fill: 'orange' },
  // hoverPaintStyle: { stroke: 'orange' },
  // endpointStyle: { width: 20, height: 16, stroke: '#666' },
  anchors: ['Right', 'Left']
};
var HIGHLIGHT_CONNECTION_OPTION = {
  // endpoint: 'Dot',
  // anchor: 'AutoDefault',
  // dragOptions: { cursor: 'pointer', zIndex: 2000 },
  paintStyle: {
    strokeWidth: 2,
    stroke: '#ffc107'
  },
  // endpointHoverStyle: { fill: 'orange' },
  // hoverPaintStyle: { stroke: 'orange' },
  endpointStyle: {
    stroke: '#ffc107'
  }
};
var DEFAULT_NODE_STYLE = {
  display: 'block',
  width: '50px',
  height: '50px',
  background: '#dfbedf',
  borderRadius: '25%',
  textAlign: 'center'
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".index-module_connect-tree__LG6Ij {\r\n  display: flex;\r\n  justify-content: space-around;\r\n  height: 100vh;\r\n  width: 100vw;\r\n  position: relative;\r\n}\r\n\r\n.index-module_connect-tree__LG6Ij .index-module_connect-tree-layer__zRYUv {\r\n  display: flex;\r\n  flex-direction: column;\r\n  justify-content: space-around;\r\n  height: 100%;\r\n}\r\n";
var style = {"connect-tree":"index-module_connect-tree__LG6Ij","connect-tree-layer":"index-module_connect-tree-layer__zRYUv"};
styleInject(css_248z);

var DefaultNodeTemplate = function DefaultNodeTemplate(props) {
  if (props.data) {
    return createElement("div", {
      className: style['connect-tree-node-content'],
      style: DEFAULT_NODE_STYLE
    }, props.data);
  } else {
    return createElement("div", null);
  }
}; // some util function ------------------------------------------------------
// generate random tree data for test


var randomGeneratesTree = function randomGeneratesTree(maxLayer, maxNodeNum, maxChildrenPerNode) {
  if (maxLayer === void 0) {
    maxLayer = 1;
  }

  if (maxNodeNum === void 0) {
    maxNodeNum = 1;
  }

  if (maxChildrenPerNode === void 0) {
    maxChildrenPerNode = 1;
  }

  var root = {
    data: 'layer 0-0',
    children: []
  };
  var totalNodeNum = 1;

  var dfs = function dfs(root, currentLayer) {
    if (root && currentLayer <= maxLayer && totalNodeNum <= maxNodeNum) {
      var childNum = 0; // random to generate a child

      var goNextRatio = Math.random();

      while (goNextRatio < 0.8 && childNum < maxChildrenPerNode && totalNodeNum < maxNodeNum) {
        // create a new node
        var child = {
          data: "layer " + currentLayer + " - " + childNum,
          children: []
        };
        totalNodeNum++; // dfs the child

        dfs(child, currentLayer + 1); // add child to current node

        if (root.children) {
          root.children.push(child);
        } else {
          root.children = [child];
        }

        childNum++;
        goNextRatio = Math.random();
      }
    }
  };

  dfs(root, 1);
  return root;
}; // generate a random and unique string

var genRandomUniqueID = function genRandomUniqueID(length) {
  return Number(Math.random().toString().substr(3, length) + Date.now()).toString(36);
}; // tranverse tree by layers


var tranverseTreeByLayers = function tranverseTreeByLayers(root) {
  var queue = []; // treeNode place by layers: layer0[treenode ...] layer1[treenode]

  var nodesByLayers = [];
  var nodesRelations = []; // start tranversing tree

  queue.push(root);

  while (queue.length > 0) {
    var numInThisLayer = queue.length;
    var nodesInThisLayer = [];

    var _loop = function _loop(i) {
      var node = queue.shift();

      if (node && node.data) {
        // add the node unique id
        node[NODE_ID_ATTR] = genRandomUniqueID(NODE_ID_LEN); // add node to this layer

        nodesInThisLayer.push(node);

        if (node.children) {
          // add children to queue (next layer)
          node.children.forEach(function (child) {
            queue.push(child);
            child[NODE_PARENT_ATTR] = node; // save the relation for connection

            nodesRelations.push({
              parent: node,
              child: child
            });
          });
        }
      }
    };

    for (var i = 0; i < numInThisLayer; i++) {
      _loop(i);
    } // complete one layer


    nodesInThisLayer && nodesByLayers.push(nodesInThisLayer);
  }

  return {
    nodesByLayers: nodesByLayers,
    nodesRelations: nodesRelations
  };
}; // the main component ----------------------------------------------------

/* TODO:
  1. batch绘制connection（节点出现1000+时，绘制线条有明显卡顿）
     --> 1000节点，不用batch时间：1500ms~6000ms；用batch时间：500~1500ms
  2. 测试遍历时间，考虑是否通过useMemo
     --> 遍历节点在1000000+才会消耗10+ms，10000节点基本在1ms。感觉没有必要useMemo?
  3. 输入数据不变时，如何避免重复渲染（node and connection）
     --> 目前datasource的引用不变不会重绘connection；使用React.memo包裹时props中引用不变不会渲染
  4. 渲染节点相当多时，考虑分片更新？
  5. 收起、展开节点？
  6. 虚拟滚动？
  7. 超过容器的线不应该绘制，容器用relative定位时scroll会错位
*/

var ConnectTree = function ConnectTree(props) {
  // props setting
  var root = props.dataSource || {};
  var connectionOption = props.connectionOption || DEFAULT_CONNECTION_OPTION;
  var highLightOption = props.highlightOption || HIGHLIGHT_CONNECTION_OPTION;
  var renderChildNode = props.renderChildNode || DefaultNodeTemplate; // tree option

  var canHighlight = props.canHighlight === false ? false : DEFAULT_TREE_OPTION.canHighlight;
  var treeStyle = props.style || {}; // hooks init

  var domRefs = useRef({});
  var plumbInstanceRef = useRef(null);

  var _useState = useState([]),
      highlightRelations = _useState[0],
      setHighlightRelations = _useState[1];

  var updateConnection = function updateConnection(realtion, highlightRelations, currentInstance, nodeRefs, connectionOption, highLightOption) {
    // clear the previos connections
    currentInstance && currentInstance.deleteEveryConnection(); // batch update connecntion for better performance

    currentInstance && currentInstance.batch(function () {
      realtion.forEach(function (_ref) {
        var parent = _ref.parent,
            child = _ref.child;

        // build new connections
        if (parent && parent[NODE_ID_ATTR] && child && child[NODE_ID_ATTR]) {
          currentInstance.connect({
            source: nodeRefs.current[parent[NODE_ID_ATTR]],
            target: nodeRefs.current[child[NODE_ID_ATTR]]
          }, connectionOption);
        }
      });
      highlightRelations.forEach(function (_ref2) {
        var parent = _ref2.parent,
            child = _ref2.child;

        // build new connections
        if (parent && parent[NODE_ID_ATTR] && child && child[NODE_ID_ATTR]) {
          currentInstance.connect({
            source: nodeRefs.current[parent[NODE_ID_ATTR]],
            target: nodeRefs.current[child[NODE_ID_ATTR]]
          }, _extends({}, connectionOption, highLightOption));
        }
      });
    });
  };

  var handleClickNode = function handleClickNode(e) {
    if (!canHighlight) return;
    var node = e.currentTarget && e.currentTarget._treeNode;
    var path = [];
    var curr = node; // find previous nodes path

    while (curr && curr[NODE_PARENT_ATTR]) {
      var parent = curr[NODE_PARENT_ATTR];
      path.push({
        parent: parent,
        child: curr
      });
      curr = parent;
    } // find next nodes path


    var dfs = function dfs(node) {
      if (!node || !node.children) {
        return;
      } else {
        node.children.forEach(function (child) {
          path.push({
            parent: node,
            child: child
          });
          dfs(child);
        });
      }
    };

    dfs(node);
    setHighlightRelations(path);
  }; // repaint all connection when window reize


  var handleResizeWindow = function handleResizeWindow() {
    var _plumbInstanceRef$cur;

    (_plumbInstanceRef$cur = plumbInstanceRef.current) == null ? void 0 : _plumbInstanceRef$cur.repaintEverything();
  }; // init plumb instance and event listener


  useEffect(function () {
    // 这里没有更新作用的状态改为useRef
    // const instance = newInstance({ ...connectionOption, container: domRefs.current['connect-tree-container'] });
    // setPlumbInstance(instance);
    plumbInstanceRef.current = newInstance(_extends({}, connectionOption, {
      container: domRefs.current['connect-tree-container']
    })); // add listener for window-resize event

    window.addEventListener('resize', handleResizeWindow);
    return function () {
      var _plumbInstanceRef$cur2;

      (_plumbInstanceRef$cur2 = plumbInstanceRef.current) == null ? void 0 : _plumbInstanceRef$cur2.destroy();
      window.removeEventListener('resize', handleResizeWindow);
    };
  }, []); // start tranverse the tree from the root

  var _tranverseTreeByLayer = tranverseTreeByLayers(root),
      nodesByLayers = _tranverseTreeByLayer.nodesByLayers,
      nodesRelations = _tranverseTreeByLayer.nodesRelations; // update connection when props(source/template) or state(plumb) change


  useEffect(function () {
    var _startTime = new Date().getTime(); //  start tranverse the tree from the root


    updateConnection(nodesRelations, highlightRelations, plumbInstanceRef.current, domRefs, connectionOption, highLightOption);

    var _endTime = new Date().getTime();

    console.log("connection update time: " + (_endTime - _startTime));
  }, [root, renderChildNode, highlightRelations, connectionOption, highLightOption]); // test

  useEffect(function () {
    console.log('tree update');
  });
  return createElement("div", {
    className: style['connect-tree'],
    style: treeStyle,
    onScroll: handleResizeWindow,
    ref: function ref(el) {
      if (el) {
        domRefs.current['connect-tree-container'] = el;
      }
    }
  }, nodesByLayers.map(function (nodesInlayer, layer) {
    // render nodes by layers
    return createElement("div", {
      className: style['connect-tree-layer'],
      key: layer
    }, nodesInlayer.map(function (node, index) {
      var node_id = node[NODE_ID_ATTR];
      return createElement("div", {
        className: style["connect-tree-node"],
        key: node_id || index,
        // give every node an unique ref
        ref: function ref(el) {
          if (node_id && el) {
            // 把node存到dom节点便于点击查找对应数据里不知是否合理
            el._treeNode = node;
            domRefs.current[node_id] = el;
          }
        },
        // 避免每个节点都创建箭头函数
        onClick: handleClickNode
      }, renderChildNode(node));
    }));
  }));
};

export default ConnectTree;
export { randomGeneratesTree, tranverseTreeByLayers };
//# sourceMappingURL=rbh-connect-tree.esm.js.map
